@@ -47,6 +47,7 @@ class HookedFS(object):
         self.repo = repo
         self.fs = fs
         self.hooks = hooks
+        self.do_make_readonly = True
         
     def __getattr__(self, name):
         return getattr(self.fs, name)
@@ -81,7 +82,15 @@ class HookedFS(object):
         self.make_readonly(filename)
         
     def make_readonly(self, filename):
-        self.fs.chmod(filename, stat.S_IRUSR)
+        if self.do_make_readonly:
+            try:
+                self.fs.chmod(filename, stat.S_IRUSR)
+            except OSError, e: # pragma: no cover
+                if e.errno != errno.EPERM:
+                    raise
+                else:
+                    logging.debug('Ignoring chmod error in repository')
+                    self.do_make_readonly = False
         
 
 class Repository(object):
@@ -117,7 +126,7 @@ class Repository(object):
 
     '''
     
-    format_version = 4
+    format_version = 5
 
     def __init__(self, fs, node_size, upload_queue_size, lru_size, hooks,
                  idpath_depth, idpath_bits, idpath_skip):
@@ -452,9 +461,9 @@ class Repository(object):
                 if other_id != gen_id:
                     other_chunks = self.client.list_chunks_in_generation(
                                         other_id)
-                    chunk_ids = [chunk_id
-                                 for chunk_id in chunk_ids
-                                 if chunk_id not in other_chunks]
+                    other_chunks = set(other_chunks)
+                    chunk_ids = chunk_ids.difference(other_chunks)
+
             return chunk_ids
 
         def remove_unused_chunks(chunk_ids):
@@ -467,7 +476,7 @@ class Repository(object):
 
         self.require_client_lock()
         logging.debug('_really_remove_generation: %d' % gen_id)
-        chunk_ids = self.client.list_chunks_in_generation(gen_id)
+        chunk_ids = set(self.client.list_chunks_in_generation(gen_id))
         chunk_ids = filter_away_chunks_used_by_other_gens(chunk_ids, gen_id)
         remove_unused_chunks(chunk_ids)
         self.client.remove_generation(gen_id)
@@ -657,3 +666,28 @@ class Repository(object):
                 return intspec
             else:
                 raise obnamlib.Error('Generation %s not found' % spec)
+
+    def walk(self, gen, arg, depth_first=False):
+        '''Iterate over each pathname specified by argument.
+        
+        This is a generator. Each return value is a tuple consisting
+        of a pathname and its corresponding metadata. Directories are
+        recursed into.
+        
+        '''
+        
+        arg = os.path.normpath(arg)
+        metadata = self.get_metadata(gen, arg)
+        if metadata.isdir():
+            if not depth_first:
+                yield arg, metadata
+            kids = self.listdir(gen, arg)
+            kidpaths = [os.path.join(arg, kid) for kid in kids]
+            for kidpath in kidpaths:
+                for x in self.walk(gen, kidpath, depth_first=depth_first):
+                    yield x
+            if depth_first:
+                yield arg, metadata
+        else:
+            yield arg, metadata
+
