@@ -17,9 +17,11 @@
 
 import errno
 import fcntl
+import grp
 import logging
 import math
 import os
+import pwd
 import tempfile
 import tracing
 
@@ -106,7 +108,53 @@ class LocalFS(obnamlib.VirtualFileSystem):
         os.rename(self.join(old), self.join(new))
 
     def lstat(self, pathname):
-        return os.lstat(self.join(pathname))
+        (ret, dev, ino, mode, nlink, uid, gid, rdev, size, blksize, blocks,
+         atime_sec, atime_nsec, mtime_sec, mtime_nsec, 
+         ctime_sec, ctime_nsec) = obnamlib._obnam.lstat(self.join(pathname))
+        if ret != 0:
+            raise OSError((ret, os.strerror(ret), pathname))
+        return obnamlib.Metadata(
+                    st_dev=dev,
+                    st_ino=ino,
+                    st_mode=mode,
+                    st_nlink=nlink,
+                    st_uid=uid,
+                    st_gid=gid,
+                    st_rdev=rdev,
+                    st_size=size,
+                    st_blksize=blksize,
+                    st_blocks=blocks,
+                    st_atime_sec=atime_sec,
+                    st_atime_nsec=atime_nsec,
+                    st_mtime_sec=mtime_sec,
+                    st_mtime_nsec=mtime_nsec,
+                    st_ctime_sec=ctime_sec,
+                    st_ctime_nsec=ctime_nsec
+                )
+
+    def get_username(self, uid):
+        return pwd.getpwuid(uid)[0]
+
+    def get_groupname(self, gid):
+        return grp.getgrgid(gid)[0]
+
+    def llistxattr(self, filename): # pragma: no cover
+        ret = obnamlib._obnam.llistxattr(self.join(filename))
+        if type(ret) is int:
+            raise OSError((ret, os.strerror(ret), filename))
+        return [s for s in ret.split('\0') if s]
+
+    def lgetxattr(self, filename, attrname): # pragma: no cover
+        ret = obnamlib._obnam.lgetxattr(self.join(filename), attrname)
+        if type(ret) is int:
+            raise OSError((ret, os.strerror(ret), filename))
+        return ret
+
+    def lsetxattr(self, filename, attrname, attrvalue): # pragma: no cover
+        ret = obnamlib._obnam.lsetxattr(self.join(filename), 
+                                        attrname, attrvalue)
+        if ret != 0:
+            raise OSError((ret, os.strerror(ret), filename))
 
     def lchown(self, pathname, uid, gid): # pragma: no cover
         tracing.trace('lchown %s %d %d', pathname, uid, gid)
@@ -116,16 +164,16 @@ class LocalFS(obnamlib.VirtualFileSystem):
         tracing.trace('chmod %s %o', pathname, mode)
         os.chmod(self.join(pathname), mode)
 
-    def lutimes(self, pathname, atime, mtime):
-        def split_time(t):
-            frac, whole = math.modf(t)
-            return int(whole), int(frac * 1e6)
-        atime_sec, atime_usec = split_time(atime)
-        mtime_sec, mtime_usec = split_time(mtime)
-        ret = obnamlib._obnam.lutimes(self.join(pathname), atime_sec,
-                                      atime_usec, mtime_sec, mtime_usec)
+    def lutimes(self, pathname, atime_sec, atime_nsec, mtime_sec, mtime_nsec):
+        assert atime_sec is not None
+        assert atime_nsec is not None
+        assert mtime_sec is not None
+        assert mtime_nsec is not None
+        ret = obnamlib._obnam.lutimes(self.join(pathname), 
+                                      atime_sec, int(atime_nsec / 1000), 
+                                      mtime_sec, int(mtime_nsec / 1000))
         if ret != 0:
-            raise OSError(ret, errno.errorcode[ret], pathname)
+            raise OSError(ret, os.strerror(ret), pathname)
 
     def link(self, existing, new):
         tracing.trace('existing=%s', existing)
@@ -196,9 +244,15 @@ class LocalFS(obnamlib.VirtualFileSystem):
         path = self.join(pathname)
         try:
             os.link(tempname, path)
-        except OSError:
-            os.remove(tempname)
-            raise
+        except OSError, e: # pragma: no cover
+            # sshfs does not implement link(2), so we fudge it here.
+            # FIXME: This has race conditions and should be made atomic.
+            if e.errno != errno.ENOSYS:
+                os.remove(tempname)
+                raise
+            if os.path.exists(path):
+                raise OSError(errno.EEXIST, os.strerror(errno.EEXIST), path)
+            os.rename(tempname, path)
         os.remove(tempname)
 
     def overwrite_file(self, pathname, contents, make_backup=True):
